@@ -32,7 +32,7 @@ typedef enum {
 	IDLE_DEBUG_TEXT = (1 << 6),
 } IdleDebugFlags;
 
-void idle_debug_init();
+void idle_debug_init (void);
 void idle_debug(IdleDebugFlags flag, const gchar *format, ...) G_GNUC_PRINTF(2, 3);
 
 void idle_debug_free (void);
