@@ -33,7 +33,7 @@
 #include "idle-text.h"
 
 static void _im_manager_iface_init(gpointer g_iface, gpointer iface_data);
-static GObject * _im_manager_constructor (GType type, guint n_props, GObjectConstructParam *props);
+static void _im_manager_constructed (GObject *obj);
 static void _im_manager_dispose (GObject *object);
 
 G_DEFINE_TYPE_WITH_CODE(IdleIMManager, idle_im_manager, G_TYPE_OBJECT,
@@ -128,7 +128,7 @@ static void idle_im_manager_class_init(IdleIMManagerClass *klass) {
 
 	g_type_class_add_private(klass, sizeof(IdleIMManagerPrivate));
 
-	object_class->constructor = _im_manager_constructor;
+	object_class->constructed = _im_manager_constructed;
 	object_class->dispose = _im_manager_dispose;
 	object_class->get_property = idle_im_manager_get_property;
 	object_class->set_property = idle_im_manager_set_property;
@@ -137,30 +137,20 @@ static void idle_im_manager_class_init(IdleIMManagerClass *klass) {
 	g_object_class_install_property(object_class, PROP_CONNECTION, param_spec);
 }
 
-static GObject *
-_im_manager_constructor (GType type,
-						 guint n_props,
-						 GObjectConstructParam *props)
+static void
+_im_manager_constructed (GObject *obj)
 {
-	GObject *obj;
-	IdleIMManager *self;
-	IdleIMManagerPrivate *priv;
+	IdleIMManager *self = IDLE_IM_MANAGER (obj);
+	IdleIMManagerPrivate *priv = IDLE_IM_MANAGER_GET_PRIVATE (self);
 
-	obj = G_OBJECT_CLASS (idle_im_manager_parent_class)->constructor (type,
-																	  n_props,
-																	  props);
+	G_OBJECT_CLASS (idle_im_manager_parent_class)->constructed (obj);
 
-	self = IDLE_IM_MANAGER (obj);
-	priv = IDLE_IM_MANAGER_GET_PRIVATE (self);
-
-	g_return_val_if_fail (priv->conn, obj);
+	g_return_if_fail (priv->conn);
 
 	priv->status_changed_id = g_signal_connect (priv->conn,
 												"status-changed", (GCallback)
 												connection_status_changed_cb,
 												self);
-
-	return obj;
 }
 
 
@@ -359,7 +349,7 @@ _im_manager_requestotron (IdleIMManager *self,
 	/* Don't support opening a channel to our self handle */
 	if (handle == base_conn->self_handle)
 	{
-		g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
+		g_set_error (&error, TP_ERROR, TP_ERROR_NOT_IMPLEMENTED,
 					 "Can't open a text channel to yourself");
 		goto error;
 	}
@@ -374,7 +364,7 @@ _im_manager_requestotron (IdleIMManager *self,
 
 	if (require_new)
 	{
-		g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+		g_set_error (&error, TP_ERROR, TP_ERROR_NOT_AVAILABLE,
 					 "Already chatting with contact #%u in another channel", handle);
 		goto error;
 	}
@@ -437,7 +427,6 @@ _im_manager_new_channel (IdleIMManager *mgr,
 		tp_base_connection_get_handles (base_connection, TP_HANDLE_TYPE_CONTACT);
 	IdleIMChannel *chan;
 	const gchar *name;
-	gchar *object_path = NULL;
 	GSList *requests = NULL;
 
 	g_assert (g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (handle))
@@ -446,14 +435,13 @@ _im_manager_new_channel (IdleIMManager *mgr,
 	name = tp_handle_inspect (handle_repo, handle);
 	IDLE_DEBUG ("Requested channel for handle: %u (%s)", handle, name);
 
-	object_path = g_strdup_printf("%s/ImChannel%u", priv->conn->parent.object_path, handle);
 	chan = g_object_new (IDLE_TYPE_IM_CHANNEL,
 						 "connection", priv->conn,
-						 "object-path", object_path,
 						 "handle", handle,
 						 "initiator-handle", initiator,
+						 "requested", handle != initiator,
 						 NULL);
-	g_free (object_path);
+	tp_base_channel_register (TP_BASE_CHANNEL (chan));
 	g_hash_table_insert (priv->channels, GUINT_TO_POINTER (handle), chan);
 
 	if (request != NULL)
