@@ -18,6 +18,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
+
 #include "idle-muc-manager.h"
 
 #include <time.h>
@@ -81,14 +83,6 @@ static void connection_status_changed_cb (IdleConnection *conn, guint status, gu
 static void _muc_manager_close_all(IdleMUCManager *manager);
 static void _muc_manager_add_handlers(IdleMUCManager *manager);
 
-static void _muc_manager_foreach_channel(TpChannelManager *manager, TpExportableChannelFunc func, gpointer user_data);
-static void _muc_manager_type_foreach_channel_class (GType type, TpChannelManagerTypeChannelClassFunc func, gpointer user_data);
-
-static gboolean _muc_manager_create_channel (TpChannelManager *manager, gpointer request_token, GHashTable *request_properties);
-static gboolean _muc_manager_request_channel (TpChannelManager *manager, gpointer request_token, GHashTable *request_properties);
-static gboolean _muc_manager_ensure_channel (TpChannelManager *manager, gpointer request_token, GHashTable *request_properties);
-static gboolean _muc_manager_request (IdleMUCManager *self, gpointer request_token, GHashTable *request_properties, gboolean require_new);
-
 static IdleMUCChannel *_muc_manager_new_channel(IdleMUCManager *manager, TpHandle handle, TpHandle initiator, gboolean requested);
 
 static void _channel_closed_cb(IdleMUCChannel *chan, gpointer user_data);
@@ -96,17 +90,28 @@ static void _channel_join_ready_cb(IdleMUCChannel *chan, guint err, gpointer use
 
 
 static const gchar * const muc_channel_fixed_properties[] = {
-    TP_IFACE_CHANNEL ".ChannelType",
-    TP_IFACE_CHANNEL ".TargetHandleType",
+    TP_PROP_CHANNEL_CHANNEL_TYPE,
+    TP_PROP_CHANNEL_TARGET_HANDLE_TYPE,
     NULL
 };
 
 static const gchar * const muc_channel_allowed_properties[] = {
-    TP_IFACE_CHANNEL ".TargetHandle",
-    TP_IFACE_CHANNEL ".TargetID",
+    TP_PROP_CHANNEL_TARGET_HANDLE,
+    TP_PROP_CHANNEL_TARGET_ID,
     NULL
 };
 
+static const gchar * const muc_channel_allowed_room_properties[] = {
+    TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, /* But it must be None */
+    TP_PROP_CHANNEL_INTERFACE_ROOM_ROOM_NAME,
+    NULL
+};
+
+static const gchar * const muc_channel_all_allowed_properties[] = {
+    TP_PROP_CHANNEL_TARGET_HANDLE,
+    TP_PROP_CHANNEL_TARGET_ID,
+    TP_PROP_CHANNEL_INTERFACE_ROOM_ROOM_NAME,
+};
 
 static GObject*
 _muc_manager_constructor(GType type, guint n_props, GObjectConstructParam *props)
@@ -181,13 +186,14 @@ static void idle_muc_manager_class_init(IdleMUCManagerClass *klass) {
 static IdleParserHandlerResult _numeric_error_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
 	IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE(user_data);
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 0));
+	IdleMUCChannel *chan;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, ignoring...");
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (!chan)
 		return IDLE_PARSER_HANDLER_RESULT_HANDLED;
@@ -221,13 +227,14 @@ static IdleParserHandlerResult _numeric_topic_handler(IdleParser *parser, IdlePa
 	IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE(user_data);
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 0));
 	const gchar *topic = g_value_get_string(g_value_array_get_nth(args, 1));
+	IdleMUCChannel *chan;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, ignoring...");
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
 		idle_muc_channel_topic(chan, topic);
@@ -240,13 +247,14 @@ static IdleParserHandlerResult _numeric_topic_stamp_handler(IdleParser *parser,
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 0));
 	TpHandle toucher_handle = g_value_get_uint(g_value_array_get_nth(args, 1));
 	time_t touched = g_value_get_uint(g_value_array_get_nth(args, 2));
+	IdleMUCChannel *chan;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, ignoring...");
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	idle_connection_emit_queued_aliases_changed(priv->conn);
 
@@ -262,6 +270,7 @@ static IdleParserHandlerResult _invite_handler(IdleParser *parser, IdleParserMes
 	TpHandle inviter_handle = g_value_get_uint(g_value_array_get_nth(args, 0));
 	TpHandle invited_handle = g_value_get_uint(g_value_array_get_nth(args, 1));
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 2));
+	IdleMUCChannel *chan;
 
 	if (invited_handle != priv->conn->parent.self_handle)
 		return IDLE_PARSER_HANDLER_RESULT_HANDLED;
@@ -271,7 +280,7 @@ static IdleParserHandlerResult _invite_handler(IdleParser *parser, IdleParserMes
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	idle_connection_emit_queued_aliases_changed(priv->conn);
 
@@ -289,6 +298,7 @@ static IdleParserHandlerResult _join_handler(IdleParser *parser, IdleParserMessa
 	IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE(manager);
 	TpHandle joiner_handle = g_value_get_uint(g_value_array_get_nth(args, 0));
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 1));
+	IdleMUCChannel *chan;
 
 	idle_connection_emit_queued_aliases_changed(priv->conn);
 
@@ -297,7 +307,7 @@ static IdleParserHandlerResult _join_handler(IdleParser *parser, IdleParserMessa
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (!chan) {
 		/* TODO: If we're in "bouncer mode", maybe these should be Requested:
@@ -316,13 +326,14 @@ static IdleParserHandlerResult _kick_handler(IdleParser *parser, IdleParserMessa
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 1));
 	TpHandle kicked_handle = g_value_get_uint(g_value_array_get_nth(args, 2));
 	const gchar *message = (args->n_values == 4) ? g_value_get_string(g_value_array_get_nth(args, 3)) : NULL;
+	IdleMUCChannel *chan;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, ignoring...");
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
 		idle_muc_channel_kick(chan, kicked_handle, kicker_handle, message);
@@ -333,13 +344,14 @@ static IdleParserHandlerResult _kick_handler(IdleParser *parser, IdleParserMessa
 static IdleParserHandlerResult _numeric_namereply_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
 	IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE(user_data);
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 0));
+	IdleMUCChannel *chan;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, ignoring...");
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
 		idle_muc_channel_namereply(chan, args);
@@ -350,13 +362,14 @@ static IdleParserHandlerResult _numeric_namereply_handler(IdleParser *parser, Id
 static IdleParserHandlerResult _numeric_namereply_end_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
 	IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE(user_data);
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 0));
+	IdleMUCChannel *chan;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, ignoring...");
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
 		idle_muc_channel_namereply_end(chan);
@@ -367,13 +380,14 @@ static IdleParserHandlerResult _numeric_namereply_end_handler(IdleParser *parser
 static IdleParserHandlerResult _mode_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
 	IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE(user_data);
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 0));
+	IdleMUCChannel *chan;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, ignoring...");
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
 		idle_muc_channel_mode(chan, args);
@@ -412,22 +426,22 @@ static IdleParserHandlerResult _notice_privmsg_handler(IdleParser *parser, IdleP
 	IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE(manager);
 	TpHandle sender_handle = (TpHandle) g_value_get_uint(g_value_array_get_nth(args, 0));
 	TpHandle room_handle = (TpHandle) g_value_get_uint(g_value_array_get_nth(args, 1));
+	IdleMUCChannel *chan;
+	TpChannelTextMessageType type;
+	gchar *body;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, ignoring...");
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 	/* XXX: just check for chan == NULL here and bail with NOT_HANDLED if room
 	 * was not found ?  Currently we go through all of the decoding of the
 	 * message, but don't actually deliver the message to a channel if chan is
 	 * NULL, and then we return 'HANDLED', which seems wrong
 	 */
 
-	TpChannelTextMessageType type;
-	gchar *body;
-
 	if (code == IDLE_PARSER_PREFIXCMD_NOTICE_CHANNEL) {
 		type = TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE;
 		body = idle_ctcp_kill_blingbling(g_value_get_string(g_value_array_get_nth(args, 2)));
@@ -451,13 +465,14 @@ static IdleParserHandlerResult _part_handler(IdleParser *parser, IdleParserMessa
 	TpHandle leaver_handle = g_value_get_uint(g_value_array_get_nth(args, 0));
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 1));
 	const gchar *message = (args->n_values == 3) ? g_value_get_string(g_value_array_get_nth(args, 2)) : NULL;
+	IdleMUCChannel *chan;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, ignoring...");
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
 		idle_muc_channel_part(chan, leaver_handle, message);
@@ -495,13 +510,14 @@ static IdleParserHandlerResult _topic_handler(IdleParser *parser, IdleParserMess
 	TpHandle room_handle = g_value_get_uint(g_value_array_get_nth(args, 1));
 	const gchar *topic = (args->n_values == 3) ? g_value_get_string(g_value_array_get_nth(args, 2)) : NULL;
 	time_t stamp = time(NULL);
+	IdleMUCChannel *chan;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, ignoring...");
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
+	chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan) {
 		if (topic)
@@ -527,9 +543,7 @@ static void _muc_manager_close_all(IdleMUCManager *manager)
 		return;
 	}
 
-	GHashTable *tmp = priv->channels;
-	priv->channels = NULL;
-	g_hash_table_destroy(tmp);
+	tp_clear_pointer (&priv->channels, g_hash_table_destroy);
 }
 
 static void
@@ -578,51 +592,50 @@ static void _muc_manager_add_handlers(IdleMUCManager *manager)
 	idle_parser_add_handler(priv->conn->parser, IDLE_PARSER_PREFIXCMD_TOPIC, _topic_handler, manager);
 }
 
-struct _ForeachHelperData {
-	TpExportableChannelFunc func;
-	gpointer user_data;
-};
+static void
+_muc_manager_foreach_channel (
+    TpChannelManager *iface,
+    TpExportableChannelFunc func,
+    gpointer user_data)
+{
+  IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE (iface);
 
-static void _foreach_helper(gpointer key, gpointer value, gpointer user_data) {
-	struct _ForeachHelperData *data = user_data;
-	data->func(value, data->user_data);
-}
+  if (priv->channels == NULL)
+    {
+      IDLE_DEBUG ("Channels hash table missing, ignoring...");
+    }
+  else
+    {
+      GHashTableIter iter;
+      gpointer v;
 
-static void _muc_manager_foreach_channel(TpChannelManager *iface, TpExportableChannelFunc func, gpointer user_data) {
-	IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE(iface);
-	struct _ForeachHelperData data = {func, user_data};
-
-	if (!priv->channels) {
-		IDLE_DEBUG("Channels hash table missing, ignoring...");
-		return;
-	}
-
-	g_hash_table_foreach(priv->channels, _foreach_helper, &data);
+      g_hash_table_iter_init (&iter, priv->channels);
+      while (g_hash_table_iter_next (&iter, NULL, &v))
+        func (TP_EXPORTABLE_CHANNEL (v), user_data);
+    }
 }
 
 static void
-_muc_manager_type_foreach_channel_class (GType type,
-									TpChannelManagerTypeChannelClassFunc func,
-									gpointer user_data)
+_muc_manager_type_foreach_channel_class (
+    GType type,
+    TpChannelManagerTypeChannelClassFunc func,
+    gpointer user_data)
 {
-	GHashTable *table = g_hash_table_new_full (g_str_hash, g_str_equal,
-											   NULL, (GDestroyNotify) tp_g_value_slice_free);
-	GValue *channel_type_value, *handle_type_value;
+  static GHashTable *handle_fixed = NULL, *room_name_fixed = NULL;
 
-	channel_type_value = tp_g_value_slice_new (G_TYPE_STRING);
-	g_value_set_static_string (channel_type_value, TP_IFACE_CHANNEL_TYPE_TEXT);
-	g_hash_table_insert (table, TP_IFACE_CHANNEL ".ChannelType",
-						 channel_type_value);
+  if (G_UNLIKELY (handle_fixed == NULL))
+    {
+      handle_fixed = tp_asv_new (
+          TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING, TP_IFACE_CHANNEL_TYPE_TEXT,
+          TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT, TP_HANDLE_TYPE_ROOM,
+          NULL);
+      room_name_fixed = tp_asv_new (
+          TP_PROP_CHANNEL_CHANNEL_TYPE, G_TYPE_STRING, TP_IFACE_CHANNEL_TYPE_TEXT,
+          NULL);
+    }
 
-	handle_type_value = tp_g_value_slice_new (G_TYPE_UINT);
-	g_value_set_uint (handle_type_value, TP_HANDLE_TYPE_ROOM);
-	g_hash_table_insert (table, TP_IFACE_CHANNEL ".TargetHandleType",
-						 handle_type_value);
-
-	func (type, table, muc_channel_allowed_properties,
-		  user_data);
-
-	g_hash_table_destroy (table);
+  func (type, handle_fixed, muc_channel_allowed_properties, user_data);
+  func (type, room_name_fixed, muc_channel_allowed_room_properties, user_data);
 }
 
 
@@ -669,7 +682,7 @@ static void _channel_closed_cb(IdleMUCChannel *chan, gpointer user_data) {
 	 * didn't finish before we killed the channel.
 	 */
 	for (GSList *l = reqs; l != NULL; l = l->next) {
-		tp_channel_manager_emit_request_failed(manager, l->data, TP_ERRORS,
+		tp_channel_manager_emit_request_failed(manager, l->data, TP_ERROR,
 			TP_ERROR_DISCONNECTED,
 			"Unable to complete this channel request, we're disconnecting!");
 	}
@@ -687,16 +700,16 @@ static void _channel_join_ready_cb(IdleMUCChannel *chan, guint err, gpointer use
 	TpChannelManager *manager = TP_CHANNEL_MANAGER(user_data);
 	IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE(user_data);
 	GSList *reqs = take_request_tokens(user_data, chan);
+	gint err_code = 0;
+	const gchar* err_msg = NULL;
+	TpHandle handle;
+	GSList *l;
 
 	if (err == MUC_CHANNEL_JOIN_ERROR_NONE) {
 		tp_channel_manager_emit_new_channel(manager, (TpExportableChannel *) chan, reqs);
 		goto out;
 	}
 
-	gint err_code = 0;
-	const gchar* err_msg = NULL;
-	TpHandle handle;
-
 	g_object_get(chan, "handle", &handle, NULL);
 
 	switch (err) {
@@ -720,8 +733,8 @@ static void _channel_join_ready_cb(IdleMUCChannel *chan, guint err, gpointer use
 			break;
 	}
 
-	for (GSList *l = reqs; reqs != NULL; reqs = reqs->next) {
-		tp_channel_manager_emit_request_failed(manager, l->data, TP_ERRORS, err_code, err_msg);
+	for (l = reqs; reqs != NULL; reqs = reqs->next) {
+		tp_channel_manager_emit_request_failed(manager, l->data, TP_ERROR, err_code, err_msg);
 	}
 
 	if (priv->channels)
@@ -731,126 +744,145 @@ out:
 	g_slist_free (reqs);
 }
 
-
 static gboolean
-_muc_manager_create_channel (TpChannelManager *manager,
-							 gpointer request_token,
-							 GHashTable *request_properties)
+_muc_manager_request (
+    IdleMUCManager *self,
+    gpointer request_token,
+    GHashTable *request_properties,
+    gboolean require_new)
 {
-	IdleMUCManager *self = IDLE_MUC_MANAGER (manager);
+  IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE (self);
+  TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
+  TpHandleRepoIface *room_repo = tp_base_connection_get_handles (base_conn,
+      TP_HANDLE_TYPE_ROOM);
+  GError *error = NULL;
+  TpHandleType handle_type;
+  TpHandle handle;
+  const gchar *channel_type;
+  IdleMUCChannel *channel;
 
-	return _muc_manager_request (self, request_token, request_properties,
-								 TRUE);
-}
+  channel_type = tp_asv_get_string (request_properties,
+      TP_PROP_CHANNEL_CHANNEL_TYPE);
 
+  if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT))
+    return FALSE;
 
-static gboolean
-_muc_manager_request_channel (TpChannelManager *manager,
-							  gpointer request_token,
-							  GHashTable *request_properties)
-{
-	IdleMUCManager *self = IDLE_MUC_MANAGER (manager);
+  handle_type = tp_asv_get_uint32 (request_properties,
+        TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, NULL);
 
-	return _muc_manager_request (self, request_token, request_properties,
-								 FALSE);
-}
+  switch (handle_type)
+    {
+      case TP_HANDLE_TYPE_ROOM:
+        handle = tp_asv_get_uint32 (request_properties,
+            TP_PROP_CHANNEL_TARGET_HANDLE, NULL);
 
+        if (!tp_handle_is_valid (room_repo, handle, &error))
+          goto error;
 
-static gboolean
-_muc_manager_ensure_channel (TpChannelManager *manager,
-							 gpointer request_token,
-							 GHashTable *request_properties)
-{
-	IdleMUCManager *self = IDLE_MUC_MANAGER (manager);
+        break;
 
-	return _muc_manager_request (self, request_token, request_properties,
-								 FALSE);
-}
+      case TP_HANDLE_TYPE_NONE:
+      {
+        const gchar *room_name = tp_asv_get_string (request_properties,
+            TP_PROP_CHANNEL_INTERFACE_ROOM_ROOM_NAME);
 
-static gboolean
-_muc_manager_request (IdleMUCManager *self,
-					  gpointer request_token,
-					  GHashTable *request_properties,
-					  gboolean require_new)
-{
-	IdleMUCManagerPrivate *priv = IDLE_MUC_MANAGER_GET_PRIVATE (self);
-	TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
-	TpHandleRepoIface *room_repo = tp_base_connection_get_handles (base_conn,
-																   TP_HANDLE_TYPE_ROOM);
-	GError *error = NULL;
-	TpHandle handle;
-	const gchar *channel_type;
-	IdleMUCChannel *channel;
+        if (room_name == NULL)
+          return FALSE;
 
-	if (tp_asv_get_uint32 (request_properties,
-						   TP_IFACE_CHANNEL ".TargetHandleType", NULL) != TP_HANDLE_TYPE_ROOM)
-		return FALSE;
+        handle = tp_handle_ensure (room_repo, room_name, NULL, &error);
+        if (handle == 0)
+          goto error;
 
-	handle = tp_asv_get_uint32 (request_properties,
-								TP_IFACE_CHANNEL ".TargetHandle", NULL);
+        break;
+      }
 
-	if (!tp_handle_is_valid (room_repo, handle, &error))
-		goto error;
+      default:
+        return FALSE;
+    }
 
-	channel_type = tp_asv_get_string (request_properties,
-									  TP_IFACE_CHANNEL ".ChannelType");
+  if (tp_channel_manager_asv_has_unknown_properties (request_properties,
+        muc_channel_fixed_properties, muc_channel_all_allowed_properties,
+        &error))
+    goto error;
 
-	if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT))
-	{
-		if (tp_channel_manager_asv_has_unknown_properties (request_properties,
-														   muc_channel_fixed_properties, muc_channel_allowed_properties,
-														   &error))
-			goto error;
+  channel = g_hash_table_lookup (priv->channels, GINT_TO_POINTER (handle));
 
-		channel = g_hash_table_lookup (priv->channels,
-										 GINT_TO_POINTER (handle));
+  if (channel != NULL)
+    {
+      if (require_new)
+        {
+          g_set_error (&error, TP_ERROR, TP_ERROR_NOT_AVAILABLE,
+                 "That channel has already been created (or requested)");
+          goto error;
+        }
+      else if (idle_muc_channel_is_ready (channel))
+        {
+          tp_channel_manager_emit_request_already_satisfied (self,
+              request_token, TP_EXPORTABLE_CHANNEL (channel));
+          return TRUE;
+        }
+    }
+  else
+    {
+      channel = _muc_manager_new_channel (self, handle, base_conn->self_handle, TRUE);
+      idle_muc_channel_join_attempt (channel);
+    }
 
-		if (channel != NULL)
-		{
-			if (require_new)
-			{
-				g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-							 "That channel has already been created (or requested)");
-				goto error;
-			}
-			else if (idle_muc_channel_is_ready (channel))
-			{
-				tp_channel_manager_emit_request_already_satisfied (self,
-																   request_token,
-																   TP_EXPORTABLE_CHANNEL (channel));
-				return TRUE;
-			}
-		}
-		else
-		{
-			channel = _muc_manager_new_channel (self, handle, base_conn->self_handle, TRUE);
-			idle_muc_channel_join_attempt(channel);
-		}
+  associate_request (self, channel, request_token);
 
-		associate_request(self, channel, request_token);
-
-		return TRUE;
-	}
-	else
-	{
-		return FALSE;
-	}
+  return TRUE;
 
 error:
-	tp_channel_manager_emit_request_failed (self, request_token,
-											error->domain, error->code, error->message);
-	g_error_free (error);
-	return TRUE;
+  tp_channel_manager_emit_request_failed (self, request_token,
+      error->domain, error->code, error->message);
+  g_error_free (error);
+  return TRUE;
 }
 
+static gboolean
+_muc_manager_create_channel (
+    TpChannelManager *manager,
+    gpointer request_token,
+    GHashTable *request_properties)
+{
+  IdleMUCManager *self = IDLE_MUC_MANAGER (manager);
 
-static void _muc_manager_iface_init(gpointer g_iface, gpointer iface_data) {
-	TpChannelManagerIface *iface = g_iface;
-
-	iface->foreach_channel = _muc_manager_foreach_channel;
-	iface->type_foreach_channel_class = _muc_manager_type_foreach_channel_class;
-	iface->request_channel = _muc_manager_request_channel;
-	iface->create_channel = _muc_manager_create_channel;
-	iface->ensure_channel = _muc_manager_ensure_channel;
+  return _muc_manager_request (self, request_token, request_properties, TRUE);
+}
+
+static gboolean
+_muc_manager_request_channel (
+  TpChannelManager *manager,
+  gpointer request_token,
+  GHashTable *request_properties)
+{
+  IdleMUCManager *self = IDLE_MUC_MANAGER (manager);
+
+  return _muc_manager_request (self, request_token, request_properties, FALSE);
+}
+
+static gboolean
+_muc_manager_ensure_channel (
+    TpChannelManager *manager,
+    gpointer request_token,
+    GHashTable *request_properties)
+{
+  IdleMUCManager *self = IDLE_MUC_MANAGER (manager);
+
+  return _muc_manager_request (self, request_token, request_properties, FALSE);
+}
+
+static void
+_muc_manager_iface_init (
+    gpointer g_iface,
+    gpointer iface_data)
+{
+  TpChannelManagerIface *iface = g_iface;
+
+  iface->foreach_channel = _muc_manager_foreach_channel;
+  iface->type_foreach_channel_class = _muc_manager_type_foreach_channel_class;
+  iface->request_channel = _muc_manager_request_channel;
+  iface->create_channel = _muc_manager_create_channel;
+  iface->ensure_channel = _muc_manager_ensure_channel;
 }
 
