@@ -21,8 +21,8 @@
 /* For strnlen(), which is a GNU extension. */
 #define  _GNU_SOURCE
 
-#include "idle-parser.h"
 #include "config.h"
+#include "idle-parser.h"
 
 #include "idle-connection.h"
 #include "idle-muc-channel.h"
@@ -180,10 +180,10 @@ static void idle_parser_finalize(GObject *obj) {
 	int i;
 
 	for (i = 0; i < IDLE_PARSER_LAST_MESSAGE_CODE; i++) {
-		GSList *link;
+		GSList *link_;
 
-		for (link = priv->handlers[i]; link != NULL; link = link->next)
-			g_slice_free(MessageHandlerClosure, link->data);
+		for (link_ = priv->handlers[i]; link_ != NULL; link_ = link_->next)
+			g_slice_free(MessageHandlerClosure, link_->data);
 
 		g_slist_free(priv->handlers[i]);
 	}
@@ -297,10 +297,10 @@ void idle_parser_remove_handlers_by_data(IdleParser *parser, gpointer user_data)
 	int i;
 
 	for (i = 0; i < IDLE_PARSER_LAST_MESSAGE_CODE; i++) {
-		GSList *link;
+		GSList *link_;
 
-		while ((link = g_slist_find_custom(priv->handlers[i], user_data, _message_handler_closure_user_data_compare)))
-			priv->handlers[i] = g_slist_remove_link(priv->handlers[i], link);
+		while ((link_ = g_slist_find_custom(priv->handlers[i], user_data, _message_handler_closure_user_data_compare)))
+			priv->handlers[i] = g_slist_remove_link(priv->handlers[i], link_);
 	}
 }
 
@@ -365,7 +365,7 @@ static void _parse_message(IdleParser *parser, const gchar *split_msg) {
 static void _parse_and_forward_one(IdleParser *parser, gchar **tokens, IdleParserMessageCode code, const gchar *format) {
 	IdleParserPrivate *priv = IDLE_PARSER_GET_PRIVATE(parser);
 	GValueArray *args = g_value_array_new(3);
-	GSList *link = priv->handlers[code];
+	GSList *link_ = priv->handlers[code];
 	IdleParserHandlerResult result = IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	gboolean success = TRUE;
 	gchar **iter = tokens;
@@ -389,23 +389,41 @@ static void _parse_and_forward_one(IdleParser *parser, gchar **tokens, IdleParse
 				iter += 2;
 			}
 		} else if ((*format == ':') || (*format == '.')) {
-			/* Because of the way things are tokenized, if there is a space
-			 * immediately after the the ':', the current token will only be
-			 * ":", so we check that iter[1][1] is non-NULL rather than checking
-			 * iter[0][1] (since iter[0] is a NULL-terminated token string
-			 * whereas iter[1] is a pointer into the full message string
+			/* Assume the happy case of the trailing parameter starting after the :
+			 * in the trailing string as the RFC intended */
+			const gchar *trailing = iter[1] + 1;
+
+			/* Some IRC proxies *cough* bip *cough* omit the : in the trailing
+			 * parameter if that parameter is just one word, to cope with that check
+			 * if there are no more tokens after the current one and if so, accept a
+			 * trailing string without the : prefix. */
+			if (iter[0][0] != ':') {
+				if (iter[2] == NULL) {
+					trailing = iter[1];
+				} else {
+					success = FALSE;
+					break;
+				}
+			}
+
+			/*
+			 * because of the way things are tokenized, if there is a
+			 * space immediately after the the ':', the current token will only be
+			 * ":", so we check that the trailing string is non-NULL rather than
+			 * checking iter[0][1] (since iter[0] is a NULL-terminated token string
+			 * whereas trailing is a pointer into the full message string
 			 */
-			if ((iter[0][0] != ':') || (iter[1][1] == '\0')) {
+			if (trailing[0] == '\0') {
 				success = FALSE;
 				break;
 			}
 
 			g_value_init(&val, G_TYPE_STRING);
-			g_value_set_string(&val, iter[1] + 1);
+			g_value_set_string(&val, trailing);
 			g_value_array_append(args, &val);
 			g_value_unset(&val);
 
-			IDLE_DEBUG("set string \"%s\"", iter[1] + 1);
+			IDLE_DEBUG("set string \"%s\"", trailing);
 		} else {
 			if (!_parse_atom(parser, args, *format, iter[0], contact_reffed, room_reffed)) {
 				success = FALSE;
@@ -431,18 +449,18 @@ static void _parse_and_forward_one(IdleParser *parser, gchar **tokens, IdleParse
 
 	IDLE_DEBUG("successfully parsed");
 
-	while (link) {
-		MessageHandlerClosure *closure = link->data;
+	while (link_) {
+		MessageHandlerClosure *closure = link_->data;
 		result = closure->handler(parser, code, args, closure->user_data);
 		if (result == IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED) {
-			link = link->next;
+			link_ = link_->next;
 		} else if (result == IDLE_PARSER_HANDLER_RESULT_HANDLED) {
 			break;
 		} else if (result == IDLE_PARSER_HANDLER_RESULT_NO_MORE_PLEASE) {
-			GSList *tmp = link->next;
+			GSList *tmp = link_->next;
 			g_free(closure);
-			priv->handlers[code] = g_slist_remove_link(priv->handlers[code], link);
-			link = tmp;
+			priv->handlers[code] = g_slist_remove_link(priv->handlers[code], link_);
+			link_ = tmp;
 		} else {
 			g_assert_not_reached();
 		}
@@ -496,12 +514,10 @@ static gboolean _parse_atom(IdleParser *parser, GValueArray *arr, char atom, con
 			if (atom == 'r') {
 				if ((handle = tp_handle_ensure(room_repo, id, NULL, NULL))) {
 					tp_handle_set_add(room_reffed, handle);
-					tp_handle_unref(room_repo, handle);
 				}
 			} else {
 				if ((handle = tp_handle_ensure(contact_repo, id, NULL, NULL))) {
 					tp_handle_set_add(contact_reffed, handle);
-					tp_handle_unref(contact_repo, handle);
 
 					idle_connection_canon_nick_receive(priv->conn, handle, id);
 				}
@@ -521,7 +537,11 @@ static gboolean _parse_atom(IdleParser *parser, GValueArray *arr, char atom, con
 
 			if (atom == 'C') {
 				g_value_init(&val, G_TYPE_CHAR);
+#if GLIB_CHECK_VERSION(2, 31, 0)
+				g_value_set_schar(&val, modechar);
+#else
 				g_value_set_char(&val, modechar);
+#endif
 				g_value_array_append(arr, &val);
 				g_value_unset(&val);
 
