@@ -31,7 +31,6 @@
 #define IDLE_DEBUG_FLAG IDLE_DEBUG_NETWORK
 #include "idle-connection.h"
 #include "idle-debug.h"
-#include "idle-server-connection-signals-marshal.h"
 
 typedef struct _IdleServerConnectionPrivate IdleServerConnectionPrivate;
 
@@ -188,7 +187,7 @@ static void idle_server_connection_class_init(IdleServerConnectionClass *klass)
 						G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
 						0,
 						NULL, NULL,
-						idle_server_connection_marshal_VOID__UINT_UINT,
+						g_cclosure_marshal_generic,
 						G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_UINT);
 
 	signals[RECEIVED] = g_signal_new("received",
@@ -224,14 +223,21 @@ static void _input_stream_read_ready(GObject *source_object, GAsyncResult *res,
 	GInputStream *input_stream = G_INPUT_STREAM(source_object);
 	IdleServerConnection *conn = IDLE_SERVER_CONNECTION(user_data);
 	IdleServerConnectionPrivate *priv = IDLE_SERVER_CONNECTION_GET_PRIVATE(conn);
+	gssize ret;
 	GError *error = NULL;
 
 	if (priv->io_stream == NULL) /* ie. we are in the process of disconnecting */
 		goto cleanup;
-	if (g_input_stream_read_finish(input_stream, res, &error) == -1) {
+
+	ret = g_input_stream_read_finish(input_stream, res, &error);
+
+	if (ret == -1) {
 		IDLE_DEBUG("g_input_stream_read failed: %s", error->message);
 		g_error_free(error);
 		goto disconnect;
+	} else if (ret == 0) {
+		IDLE_DEBUG("g_input_stream_read returned end-of-file");
+		goto disconnect;
 	}
 
 	g_signal_emit(conn, signals[RECEIVED], 0, priv->input_buffer);
@@ -252,7 +258,7 @@ static void _connect_to_host_ready(GObject *source_object, GAsyncResult *res, gp
 	IdleServerConnection *conn = IDLE_SERVER_CONNECTION(g_async_result_get_source_object(G_ASYNC_RESULT(result)));
 	IdleServerConnectionPrivate *priv = IDLE_SERVER_CONNECTION_GET_PRIVATE(conn);
 	GInputStream *input_stream;
-	GSocket *socket;
+	GSocket *socket_;
 	GSocketConnection *socket_connection;
 	gint nodelay = 1;
 	gint socket_fd;
@@ -261,17 +267,17 @@ static void _connect_to_host_ready(GObject *source_object, GAsyncResult *res, gp
 	socket_connection = g_socket_client_connect_to_host_finish(socket_client, res, &error);
 	if (socket_connection == NULL) {
 		IDLE_DEBUG("g_socket_client_connect_to_host failed: %s", error->message);
-		g_simple_async_result_set_error(result, TP_ERRORS, TP_ERROR_NETWORK_ERROR, "%s", error->message);
+		g_simple_async_result_set_error(result, TP_ERROR, TP_ERROR_NETWORK_ERROR, "%s", error->message);
 		g_error_free(error);
 		change_state(conn, SERVER_CONNECTION_STATE_NOT_CONNECTED, SERVER_CONNECTION_STATE_REASON_ERROR);
 		g_object_unref(conn);
 		goto cleanup;
 	}
 
-	socket = g_socket_connection_get_socket(socket_connection);
-	g_socket_set_keepalive(socket, TRUE);
+	socket_ = g_socket_connection_get_socket(socket_connection);
+	g_socket_set_keepalive(socket_, TRUE);
 
-	socket_fd = g_socket_get_fd(socket);
+	socket_fd = g_socket_get_fd(socket_);
 	setsockopt(socket_fd, IPPROTO_TCP, TCP_NODELAY, &nodelay, sizeof(nodelay));
 
 	g_tcp_connection_set_graceful_disconnect(G_TCP_CONNECTION(socket_connection), TRUE);
@@ -295,7 +301,7 @@ void idle_server_connection_connect_async(IdleServerConnection *conn, GCancellab
 		IDLE_DEBUG("already connecting or connected!");
 		g_simple_async_report_error_in_idle(G_OBJECT(conn),
 			callback, user_data,
-			TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+			TP_ERROR, TP_ERROR_NOT_AVAILABLE,
 			"already connecting or connected!");
 		return;
 	}
@@ -304,7 +310,7 @@ void idle_server_connection_connect_async(IdleServerConnection *conn, GCancellab
 		IDLE_DEBUG("host not set!");
 		g_simple_async_report_error_in_idle(G_OBJECT(conn),
 			callback, user_data,
-			TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+			TP_ERROR, TP_ERROR_NOT_AVAILABLE,
 			"host not set!");
 		return;
 	}
@@ -313,7 +319,7 @@ void idle_server_connection_connect_async(IdleServerConnection *conn, GCancellab
 		IDLE_DEBUG("port not set!");
 		g_simple_async_report_error_in_idle(G_OBJECT(conn),
 			callback, user_data,
-			TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+			TP_ERROR, TP_ERROR_NOT_AVAILABLE,
 			"port not set!");
 		return;
 	}
@@ -340,7 +346,7 @@ static void _close_ready(GObject *source_object, GAsyncResult *res, gpointer use
 
 	if (!g_io_stream_close_finish(io_stream, res, &error)) {
 		IDLE_DEBUG("g_io_stream_close failed: %s", error->message);
-		g_simple_async_result_set_error(result, TP_ERRORS, TP_ERROR_NETWORK_ERROR, "%s", error->message);
+		g_simple_async_result_set_error(result, TP_ERROR, TP_ERROR_NETWORK_ERROR, "%s", error->message);
 		g_error_free(error);
 	}
 
@@ -366,11 +372,20 @@ void idle_server_connection_disconnect_full_async(IdleServerConnection *conn, gu
 		IDLE_DEBUG("the connection was not open");
 		g_simple_async_report_error_in_idle(G_OBJECT(conn),
 			callback, user_data,
-			TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+			TP_ERROR, TP_ERROR_NOT_AVAILABLE,
 			"the connection was not open");
 		return;
 	}
 
+	if (priv->io_stream == NULL) {
+		IDLE_DEBUG("We were exploding anyway");
+		g_simple_async_report_error_in_idle(G_OBJECT(conn),
+			callback, user_data,
+			TP_ERROR, TP_ERROR_NOT_AVAILABLE,
+			"We were exploding anyway");
+		return;
+	}
+
 	priv->reason = reason;
 
 	result = g_simple_async_result_new(G_OBJECT(conn), callback, user_data, idle_server_connection_disconnect_full_async);
@@ -397,7 +412,7 @@ static void _write_ready(GObject *source_object, GAsyncResult *res, gpointer use
 	nwrite = g_output_stream_write_finish(output_stream, res, &error);
 	if (nwrite == -1) {
 		IDLE_DEBUG("g_output_stream_write failed : %s", error->message);
-		g_simple_async_result_set_error(result, TP_ERRORS, TP_ERROR_NETWORK_ERROR, "%s", error->message);
+		g_simple_async_result_set_error(result, TP_ERROR, TP_ERROR_NETWORK_ERROR, "%s", error->message);
 		g_error_free(error);
 		goto cleanup;
 	}
@@ -423,11 +438,12 @@ void idle_server_connection_send_async(IdleServerConnection *conn, const gchar *
 	GSimpleAsyncResult *result;
 	gsize output_buffer_size = sizeof(priv->output_buffer);
 
-	if (priv->state != SERVER_CONNECTION_STATE_CONNECTED) {
+	if (priv->state != SERVER_CONNECTION_STATE_CONNECTED
+            || priv->io_stream == NULL) {
 		IDLE_DEBUG("connection was not open!");
 		g_simple_async_report_error_in_idle(G_OBJECT(conn),
 			callback, user_data,
-			TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+			TP_ERROR, TP_ERROR_NOT_AVAILABLE,
 			"connection was not open!");
 		return;
 	}
@@ -457,7 +473,9 @@ void idle_server_connection_send_async(IdleServerConnection *conn, const gchar *
 }
 
 gboolean idle_server_connection_send_finish(IdleServerConnection *conn, GAsyncResult *result, GError **error) {
-	g_return_val_if_fail(g_simple_async_result_is_valid(result, G_OBJECT(conn), idle_server_connection_send_async), FALSE);
+	/* we can't check the source tag here because we use
+	 * report_error_in_idle which never sets a source tag */
+	g_return_val_if_fail(g_simple_async_result_is_valid(result, G_OBJECT(conn), NULL), FALSE);
 	return !g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT(result), error);
 }
 
