@@ -3,7 +3,9 @@
 #include <stdio.h>
 #include <string.h>
 
-int main() {
+int
+main (void)
+{
 	gboolean fail = FALSE;
 
 	const gchar *test_strings[] = {
